"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GmailTransport = void 0;
const Requestly_1 = require("./services/Requestly");
function refreshTokenParams(auth) {
    return {
        client_id: auth.clientId,
        client_secret: auth.clientSecret,
        refresh_token: auth.refreshToken,
        grant_type: 'refresh_token',
    };
}
function getErrorCode(error) {
    return typeof error === 'object' && error !== null ? error.code : 0;
}
function createError(ge) {
    if (!(typeof ge === 'object' && ge !== null))
        return new Error(ge);
    if (ge.error === 'invalid_grant') {
        // return a better message instead of 'Bad Request' from the Gmail error_description.
        return new Error('Invalid grant. Please reconnect your Gmail account');
    }
    if (ge.error_description) {
        return new Error(ge.error_description);
    }
    if (typeof ge.error === 'object' && ge.error !== null) {
        return new Error(ge.error.message);
    }
    return new Error(ge.error);
}
class GmailTransport {
    constructor(options) {
        this.options = options;
        this.name = 'GmailTransport';
        this.version = 'N/A';
    }
    getAccessToken() {
        return (0, Requestly_1.post)({
            protocol: 'https:',
            hostname: 'www.googleapis.com',
            path: '/oauth2/v4/token',
        }, refreshTokenParams(this.options.auth)).then((data) => {
            if (typeof data === 'string') {
                return Promise.reject(data);
            }
            return data.access_token;
        });
    }
    sendMail(message, accessToken) {
        return (0, Requestly_1.postRFC822)({
            protocol: 'https:',
            hostname: 'www.googleapis.com',
            path: `/upload/gmail/v1/users/${this.options.userId || 'me'}/messages/send?uploadType=multipart`,
            headers: {
                Authorization: `Bearer ${accessToken}`,
            },
        }, message.toString());
    }
    send(mail, done) {
        mail.message.keepBcc = true;
        mail.message.build((error, data) => {
            if (error)
                return done(error);
            if (!data)
                return done(new Error('The email data is corrapted.'));
            this.sendMail(data, this.options.auth.accessToken).then((message) => {
                done(null, {
                    envelope: mail.message.getEnvelope(),
                    messageId: mail.message.messageId(),
                    accessToken: this.options.auth.accessToken,
                    message: message,
                });
            }, (err) => {
                if (getErrorCode(err.error) === 401 && this.options.auth.refreshToken) {
                    this.getAccessToken()
                        .then((accessToken) => {
                        this.sendMail(data, accessToken).then((message) => {
                            done(null, {
                                envelope: mail.message.getEnvelope(),
                                messageId: mail.message.messageId(),
                                accessToken: accessToken,
                                message: message,
                            });
                        }, (err) => done(createError(err)));
                    })
                        .catch((err) => done(createError(err)));
                }
                else {
                    done(createError(err));
                }
            });
        });
    }
}
exports.GmailTransport = GmailTransport;
exports.default = GmailTransport;
