"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postRFC822 = exports.post = void 0;
const url_1 = require("url");
const https_1 = require("https");
function parseJSON(rawData) {
    try {
        return JSON.parse(rawData);
    }
    catch (ignore) {
        return rawData;
    }
}
function makeParams(params) {
    return new url_1.URLSearchParams(params).toString();
}
function sendRequest(options, params) {
    return new Promise((resolve, reject) => {
        const req = (0, https_1.request)(options, (res) => {
            const chunks = [];
            res.on('data', (chunk) => chunks.push(chunk));
            res.on('end', () => {
                const data = parseJSON(Buffer.concat(chunks).toString());
                if (res.statusCode === 200) {
                    resolve(data);
                }
                else {
                    reject(data);
                }
            });
            res.on('error', (error) => {
                reject(error);
            });
        });
        req.on('error', (error) => {
            reject(error);
        });
        if (params) {
            req.write(params);
        }
        req.end();
    });
}
function post(options, params) {
    const postData = makeParams(params);
    options.method = 'POST';
    options.headers = {
        'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
        'Content-Length': Buffer.byteLength(postData),
    };
    return sendRequest(options, postData);
}
exports.post = post;
function postRFC822(options, data) {
    options.method = 'POST';
    options.headers = Object.assign(options.headers || {}, {
        'Content-Type': 'message/rfc822',
        'Content-Length': Buffer.byteLength(data),
    });
    return sendRequest(options, data);
}
exports.postRFC822 = postRFC822;
